import os
import json
from datetime import datetime, timedelta
import requests

def get_valid_token():
    token_path = 'path_to_token_file.json'
    try:
        with open(token_path, 'r') as file:
            token_data = json.load(file)
            expiration_str = token_data.get('expiration')
            expiration = datetime.strptime(expiration_str, "%Y-%m-%d %H:%M:%S")
            if expiration > datetime.now():
                return token_data['access_token']
            else:
                return refresh_token()
    except (FileNotFoundError, json.JSONDecodeError):
        return refresh_token()

def refresh_token():
    url = 'https://api.koreainvestment.com/oauth2/tokenP'
    data = {
        'grant_type': 'client_credentials',
        'appkey': os.getenv('MY_APP'),
        'appsecret': os.getenv('MY_SEC')
    }
    headers = {'Content-Type': 'application/json'}

    response = requests.post(url, headers=headers, json=data)
    if response.status_code == 200:
        response_data = response.json()
        new_token = response_data['access_token']
        expires_in = int(response_data['access_token_token_expired'])
        expiration = datetime.now() + timedelta(seconds=expires_in)
        token_data = {
            'access_token': new_token,
            'expiration': expiration.strftime("%Y-%m-%d %H:%M:%S")
        }
        with open('path_to_token_file.json', 'w') as file:
            json.dump(token_data, file)
        return new_token
    else:
        raise Exception("Failed to refresh token")
