import time
from data_collection import update_data
from data_preprocessing import preprocess_data
from feature_engineering import add_features
from model_training import train_model, update_model
from prediction import predict
from live_data_processing import real_time_prediction, update_real_time_model
from backtesting import backtest_strategy
from hyperparameter_tuning import hyperparameter_tuning
from config import *

def feature_engineering():
    stock_data, news_data, financial_data, economic_data = update_data()
    stock_data = add_features(stock_data)
    print(stock_data.head())

def hyperparameter_tuning_main():
    stock_data, news_data, financial_data, economic_data = update_data()
    stock_data = add_features(stock_data)
    (train_data, test_data), scaler = preprocess_data(stock_data, financial_data, news_data, economic_data)
    time_step = 60
    X_train, y_train = create_dataset(train_data.values, time_step)
    best_params = hyperparameter_tuning(X_train, y_train)
    print("Best Parameters:", best_params)

def backtesting_main():
    stock_data, news_data, financial_data, economic_data = update_data()
    stock_data = add_features(stock_data)
    (train_data, test_data), scaler = preprocess_data(stock_data, financial_data, news_data, economic_data)
    model = train_model(train_data['Close'])
    predictions, y_test = predict(model, test_data['Close'], scaler)
    cumulative_returns = backtest_strategy(test_data, predictions)
    print("Cumulative Returns:", cumulative_returns)

def live_data_processing():
    stock_data, news_data, financial_data, economic_data = update_data()
    stock_data = add_features(stock_data)
    (train_data, test_data), scaler = preprocess_data(stock_data, financial_data, news_data, economic_data)
    model = train_model(train_data['Close'])

    while True:
        real_time_predictions = real_time_prediction(model, scaler, STOCK_TICKER)
        print(real_time_predictions)
        time.sleep(UPDATE_INTERVAL)
        stock_data, news_data, financial_data, economic_data = update_data()
        stock_data = add_features(stock_data)
        (train_data, test_data), scaler = preprocess_data(stock_data, financial_data, news_data, economic_data)
        model = update_real_time_model(model, scaler, STOCK_TICKER)

if __name__ == '__main__':
    # 주석 해제하여 실행할 함수를 선택하세요.
    # feature_engineering()
    # hyperparameter_tuning_main()
    # backtesting_main()
    # live_data_processing()
    pass
