import asyncio
import websockets
import json
import logging

async def handle_client(websocket, path):
    logging.info("Client connected")
    try:
        async for message in websocket:
            logging.info(f"Received message: {message}")
            await process_message(message)
    except websockets.ConnectionClosed as e:
        logging.info(f"Connection closed: {e}")
    except Exception as e:
        logging.error(f"Error handling client: {e}")

async def process_message(message):
    try:
        message_data = json.loads(message)
        if 'header' in message_data and message_data['header']['tr_id'] == 'PINGPONG':
            logging.info("Received a ping-pong message. No action needed.")
            return

        if 'body' in message_data and 'output' in message_data['body']:
            stock_code = message_data['body']['output'].get('stock_code')
            if stock_code:
                logging.info(f"Processing stock code: {stock_code}")
                # Process stock data here
            else:
                logging.warning("No stock code found in the message")
        else:
            logging.warning("Unexpected message format")
    except json.JSONDecodeError:
        logging.error("Error decoding JSON message")

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    start_server = websockets.serve(handle_client, "localhost", 21000)
    asyncio.get_event_loop().run_until_complete(start_server)
    logging.info("Websocket server started")
    asyncio.get_event_loop().run_forever()
